
# This file was generated by 'versioneer.py' (0.19) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2022-01-18T08:29:04+0000",
 "dirty": false,
 "error": null,
 "full-revisionid": "0df5b0e55d3159bbd4977d7f18b7a2a4a230543a",
 "version": "1.0.0-rc.1"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
